import datetime
import importlib

from celery import shared_task
from django.conf import settings
from infi.clickhouse_orm.utils import import_submodules

from django_clickhouse.clickhouse_models import ClickHouseModel
from .configuration import config
from .utils import get_subclasses


@shared_task(queue=config.CELERY_QUEUE)
def sync_clickhouse_model(model_cls) -> None:
    """
    Syncs one batch of given ClickHouseModel
    :param model_cls: ClickHouseModel subclass
    :return: None
    """
    model_cls.get_storage().set_last_sync_time(model_cls.get_import_key(), datetime.datetime.now())
    model_cls.sync_batch_from_storage()


@shared_task(queue=config.CELERY_QUEUE)
def clickhouse_auto_sync():
    """
    Plans syncing models
    :return: None
    """
    # Import all model modules
    for app in settings.INSTALLED_APPS:
        package_name = "%s.%s" % (app, config.MODELS_MODULE)
        try:
            module = importlib.import_module(package_name)
            if hasattr(module, '__path__'):
                import_submodules(package_name)
        except ImportError:
            pass

    # Start
    for cls in get_subclasses(ClickHouseModel, recursive=True):
        if cls.need_sync():
            # Даже если синхронизация вдруг не выполнится, не страшно, что мы установили период синхронизации
            # Она выполнится следующей таской через интервал.
            sync_clickhouse_model.delay(cls)



"""
This file defines router to find appropriate database
"""
from typing import Type

import random
import six
from infi.clickhouse_orm.migrations import Operation, DropTable, CreateTable

from .clickhouse_models import ClickHouseModel
from .configuration import config
from .utils import lazy_class_import


class DefaultRouter:
    def db_for_read(self, model: Type[ClickHouseModel], **hints) -> str:
        """
        Gets database to read from for model
        :param model: Model to decide for
        :param hints: Some hints to make correct choice
        :return: Database alias
        """
        return random.choice(model.read_db_aliases)

    def db_for_write(self, model: Type[ClickHouseModel], **hints) -> str:
        """
        Gets database to write to for model
        :param model: Model to decide for
        :param hints: Some hints to make correct choice
        :return: Database alias
        """
        return random.choice(model.write_db_aliases)

    def allow_migrate(self, db_alias: str, app_label: str, operation: Operation,
                      model=None, **hints) -> bool:
        """
        Checks if migration can be applied to given database
        :param db_alias: Database alias to check
        :param app_label: App from which migration is got
        :param operation: Operation object to perform
        :param model: Model migration is applied to
        :param hints: Hints to make correct decision
        :return: boolean
        """
        if hints.get("force_migrate_on_databases", None):
            return db_alias in hints["force_migrate_on_databases"]

        if hints.get('model'):
            model = '%s.%s.%s' % (app_label, config.MODELS_MODULE, hints['model']) \
                if isinstance(hints['model'], six.string_types) else hints['model']

        model = lazy_class_import(model)

        if operation.__class__ not in {CreateTable, DropTable}:
            return db_alias in model.migrate_replicated_db_aliases
        else:
            return db_alias in model.migrate_non_replicated_db_aliases
